@@ -5,7 +5,7 @@
 
     sphinx-build command-line handling.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import print_function
@@ -16,16 +16,22 @@ import traceback
 from os import path
 
 from six import text_type, binary_type
+
 from docutils.utils import SystemMessage
 
 from sphinx import __display_version__
 from sphinx.errors import SphinxError
 from sphinx.application import Sphinx
 from sphinx.util import Tee, format_exception_cut_frames, save_traceback
-from sphinx.util.console import red, nocolor, color_terminal
+from sphinx.util.console import red, nocolor, color_terminal  # type: ignore
+from sphinx.util.docutils import docutils_namespace
 from sphinx.util.osutil import abspath, fs_encoding
 from sphinx.util.pycompat import terminal_safe
 
+if False:
+    # For type annotation
+    from typing import Any, IO, List, Union  # NOQA
+
 
 USAGE = """\
 Sphinx v%s
@@ -44,21 +50,72 @@ For more information, visit <http://sphinx-doc.org/>.
 
 class MyFormatter(optparse.IndentedHelpFormatter):
     def format_usage(self, usage):
+        # type: (Any) -> Any
         return usage
 
     def format_help(self, formatter):
-        result = []
-        if self.description:
+        # type: (Any) -> unicode
+        result = []  # type: List[unicode]
+        if self.description:  # type: ignore
             result.append(self.format_description(formatter))
-        if self.option_list:
-            result.append(self.format_option_help(formatter))
+        if self.option_list:  # type: ignore
+            result.append(self.format_option_help(formatter))  # type: ignore
         return "\n".join(result)
 
 
-def main(argv):
-    if not color_terminal():
-        nocolor()
+def handle_exception(app, opts, exception, stderr=sys.stderr):
+    # type: (Sphinx, Any, Union[Exception, KeyboardInterrupt], IO) -> None
+    if opts.pdb:
+        import pdb
+        print(red('Exception occurred while building, starting debugger:'),
+              file=stderr)
+        traceback.print_exc()
+        pdb.post_mortem(sys.exc_info()[2])
+    else:
+        print(file=stderr)
+        if opts.verbosity or opts.traceback:
+            traceback.print_exc(None, stderr)
+            print(file=stderr)
+        if isinstance(exception, KeyboardInterrupt):
+            print('interrupted!', file=stderr)
+        elif isinstance(exception, SystemMessage):
+            print(red('reST markup error:'), file=stderr)
+            print(terminal_safe(exception.args[0]), file=stderr)
+        elif isinstance(exception, SphinxError):
+            print(red('%s:' % exception.category), file=stderr)
+            print(terminal_safe(text_type(exception)), file=stderr)
+        elif isinstance(exception, UnicodeError):
+            print(red('Encoding error:'), file=stderr)
+            print(terminal_safe(text_type(exception)), file=stderr)
+            tbpath = save_traceback(app)
+            print(red('The full traceback has been saved in %s, if you want '
+                      'to report the issue to the developers.' % tbpath),
+                  file=stderr)
+        elif isinstance(exception, RuntimeError) and 'recursion depth' in str(exception):
+            print(red('Recursion error:'), file=stderr)
+            print(terminal_safe(text_type(exception)), file=stderr)
+            print(file=stderr)
+            print('This can happen with very large or deeply nested source '
+                  'files.  You can carefully increase the default Python '
+                  'recursion limit of 1000 in conf.py with e.g.:', file=stderr)
+            print('    import sys; sys.setrecursionlimit(1500)', file=stderr)
+        else:
+            print(red('Exception occurred:'), file=stderr)
+            print(format_exception_cut_frames().rstrip(), file=stderr)
+            tbpath = save_traceback(app)
+            print(red('The full traceback has been saved in %s, if you '
+                      'want to report the issue to the developers.' % tbpath),
+                  file=stderr)
+            print('Please also report this if it was a user error, so '
+                  'that a better error message can be provided next time.',
+                  file=stderr)
+            print('A bug report can be filed in the tracker at '
+                  '<https://github.com/sphinx-doc/sphinx/issues>. Thanks!',
+                  file=stderr)
 
+
+def main(argv):
+    # type: (List[unicode]) -> int
     parser = optparse.OptionParser(USAGE, epilog=EPILOG, formatter=MyFormatter())
     parser.add_option('--version', action='store_true', dest='version',
                       help='show version information and exit')
@@ -107,8 +164,12 @@ def main(argv):
                      help='no output on stdout, just warnings on stderr')
     group.add_option('-Q', action='store_true', dest='really_quiet',
                      help='no output at all, not even warnings')
-    group.add_option('-N', action='store_true', dest='nocolor',
-                     help='do not emit colored output')
+    group.add_option('--color', dest='color',
+                     action='store_const', const='yes', default='auto',
+                     help='Do emit colored output (default: auto-detect)')
+    group.add_option('-N', '--no-color', dest='color',
+                     action='store_const', const='no',
+                     help='Do not emit colored output (default: auot-detect)')
     group.add_option('-w', metavar='FILE', dest='warnfile',
                      help='write warnings (and errors) to given file')
     group.add_option('-W', action='store_true', dest='warningiserror',
@@ -159,12 +220,12 @@ def main(argv):
 
     # handle remaining filename arguments
     filenames = args[2:]
-    err = 0
+    errored = False
     for filename in filenames:
         if not path.isfile(filename):
             print('Error: Cannot find file %r.' % filename, file=sys.stderr)
-            err = 1
-    if err:
+            errored = True
+    if errored:
         return 1
 
     # likely encoding used for command-line arguments
@@ -178,7 +239,7 @@ def main(argv):
         print('Error: Cannot combine -a option and filenames.', file=sys.stderr)
         return 1
 
-    if opts.nocolor:
+    if opts.color == 'no' or (opts.color == 'auto' and not color_terminal()):
         nocolor()
 
     doctreedir = abspath(opts.doctreedir or path.join(outdir, '.doctrees'))
@@ -198,7 +259,7 @@ def main(argv):
             print('Error: Cannot open warning file %r: %s' %
                   (opts.warnfile, exc), file=sys.stderr)
             sys.exit(1)
-        warning = Tee(warning, warnfp)
+        warning = Tee(warning, warnfp)  # type: ignore
         error = warning
 
     confoverrides = {}
@@ -238,57 +299,12 @@ def main(argv):
 
     app = None
     try:
-        app = Sphinx(srcdir, confdir, outdir, doctreedir, opts.builder,
-                     confoverrides, status, warning, opts.freshenv,
-                     opts.warningiserror, opts.tags, opts.verbosity, opts.jobs)
-        app.build(opts.force_all, filenames)
-        return app.statuscode
-    except (Exception, KeyboardInterrupt) as err:
-        if opts.pdb:
-            import pdb
-            print(red('Exception occurred while building, starting debugger:'),
-                  file=error)
-            traceback.print_exc()
-            pdb.post_mortem(sys.exc_info()[2])
-        else:
-            print(file=error)
-            if opts.verbosity or opts.traceback:
-                traceback.print_exc(None, error)
-                print(file=error)
-            if isinstance(err, KeyboardInterrupt):
-                print('interrupted!', file=error)
-            elif isinstance(err, SystemMessage):
-                print(red('reST markup error:'), file=error)
-                print(terminal_safe(err.args[0]), file=error)
-            elif isinstance(err, SphinxError):
-                print(red('%s:' % err.category), file=error)
-                print(terminal_safe(text_type(err)), file=error)
-            elif isinstance(err, UnicodeError):
-                print(red('Encoding error:'), file=error)
-                print(terminal_safe(text_type(err)), file=error)
-                tbpath = save_traceback(app)
-                print(red('The full traceback has been saved in %s, if you want '
-                          'to report the issue to the developers.' % tbpath),
-                      file=error)
-            elif isinstance(err, RuntimeError) and 'recursion depth' in str(err):
-                print(red('Recursion error:'), file=error)
-                print(terminal_safe(text_type(err)), file=error)
-                print(file=error)
-                print('This can happen with very large or deeply nested source '
-                      'files.  You can carefully increase the default Python '
-                      'recursion limit of 1000 in conf.py with e.g.:', file=error)
-                print('    import sys; sys.setrecursionlimit(1500)', file=error)
-            else:
-                print(red('Exception occurred:'), file=error)
-                print(format_exception_cut_frames().rstrip(), file=error)
-                tbpath = save_traceback(app)
-                print(red('The full traceback has been saved in %s, if you '
-                          'want to report the issue to the developers.' % tbpath),
-                      file=error)
-                print('Please also report this if it was a user error, so '
-                      'that a better error message can be provided next time.',
-                      file=error)
-                print('A bug report can be filed in the tracker at '
-                      '<https://github.com/sphinx-doc/sphinx/issues>. Thanks!',
-                      file=error)
-            return 1
+        with docutils_namespace():
+            app = Sphinx(srcdir, confdir, outdir, doctreedir, opts.builder,
+                         confoverrides, status, warning, opts.freshenv,
+                         opts.warningiserror, opts.tags, opts.verbosity, opts.jobs)
+            app.build(opts.force_all, filenames)
+            return app.statuscode
+    except (Exception, KeyboardInterrupt) as exc:
+        handle_exception(app, opts, exc, error)
+        return 1
