@@ -5,12 +5,16 @@
 
     A Base class for additional parsers.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import docutils.parsers
 
+if False:
+    # For type annotation
+    from sphinx.application import Sphinx  # NOQA
+
 
 class Parser(docutils.parsers.Parser):
     """
@@ -33,6 +37,7 @@ class Parser(docutils.parsers.Parser):
     """
 
     def set_application(self, app):
+        # type: (Sphinx) -> None
         """set_application will be called from Sphinx to set app and other instance variables
 
         :param sphinx.application.Sphinx app: Sphinx application object
